package com.biganto.visual.roompark.data.repository.db.requrey

import android.content.Context
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.FeedEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.Models
import com.biganto.visual.roompark.data.repository.db.requrey.model.UserEntity
import com.biganto.visual.roompark.di.dagger.DATABASE_VERSION
import dagger.Module
import dagger.Provides
import io.reactivex.Observable
import io.reactivex.Single
import io.requery.Persistable
import io.requery.android.sqlite.DatabaseSource
import io.requery.reactivex.KotlinReactiveEntityStore
import io.requery.sql.KotlinEntityDataStore
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 13.06.2018.
 */


@Module
class DbModule{

        @Provides
        fun provideStore(context: Context): KotlinReactiveEntityStore<Persistable> {
                Timber.d("Kotlin store creating..")
                val source = DatabaseSource(context, Models.DEFAULT, "BigantoPerfect", DATABASE_VERSION)

                source.setLoggingEnabled(false)

                val store = KotlinEntityDataStore<Persistable>(source.configuration)
                Timber.d("Kotlin store %s",source)
                return KotlinReactiveEntityStore(store)
        }
}

class RequeryRepository @Inject constructor(
        private val store: KotlinReactiveEntityStore<Persistable>
)
: IDb {

        private inline fun <reified T:Persistable> fetchAll() =
                store.select(T::class)

        override fun upsertUser(entity:UserEntity): Observable<UserEntity> =
                store.upsert(entity).toObservable()

        override fun <T:Persistable> upsert(entity:T) = store.upsert(entity)

        override fun <T : List<Persistable>> upsert(entity: T): Single<Iterable<Persistable>> =
                store.upsert(entity)

        override fun <T:List<Persistable>> blockingUpsert(entity:T) =
                store.upsert(entity).toObservable().blockingSubscribe()


        override fun fetchFeeds(): Observable<FeedEntity> =
                fetchAll<FeedEntity>().get().observable()

        override fun fetchUser(uuid:Int): Observable<UserEntity> =
                fetchAll<UserEntity>().where(UserEntity.UUID.eq(uuid)).get().observable()


        fun upsertFeeds(entity:List<FeedEntity>) =
                store.upsert(entity)

}